# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template2LevelModAML(Template):
    'Fits 2-level random intercept models for Normal, Poisson, binary, binomial or negative binomial distributions, using aML only.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'Binomial', 'Negative binomial', 'Poisson', 'Probit', 'Logit' ]
    engines = ['aML']

    inputs = '''
ID = IDVector('Identifier variable')
y = DataVector('Response: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Binary', 'Poisson', 'Negative Binomial'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    link = Text('Specify link function: ', ['logit', 'probit'])
if D == 'Binary':
    link = Text('Specify link function: ', ['logit', 'probit'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
if D == 'Normal':
    sigma = ParamScalar()
if D == 'Negative Binomial':
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
sigmau = ParamScalar()
x = DataMatrix('Explanatory variables: ', allow_cat = True)
beta = ParamVector(parents=[x], as_scalar=True)
'''
        
    amlscript = '''

option maximum number of residual draws = 200;

define regressor set BetaX;
var = \\
% for p in x:
${p} \\
% endfor
;


define normal distribution; dim=1;
    name=u;

% if D == 'Normal':
define normal distribution; dim=1;
    name=e;

continuous model;
    outcome = ${y};
    model = regset BetaX +
        res(draw=1, ref=u) + res(draw=_iid, ref=e);
% endif

% if D == 'Binomial':
binomial model;
    outcome = ${y};
    exposure = ${n};
% if link == 'logit':
    probability = logistic(regset BetaX + intres(draw=1, ref=u));
% endif
% if link == 'probit':
    probability = normprob(regset BetaX + intres(draw=1, ref=u));
% endif
% endif
% if D == 'Binary':
% if link == 'logit':
logit model;
% endif
% if link == 'probit':
probit model;
% endif
  outcome=use;
  model = regset BetaX + intres(draw=1, ref=u);
% endif
% if D == 'Poisson':
poisson model;
    outcome = ${y};
% if offset:
    incidence = exp(${n} + regset BetaX + intres(draw=1, ref=u));
% else:
    incidence = exp(regset BetaX + intres(draw=1, ref=u));
% endif
% endif
% if D == 'Negative Binomial':
define parameter Alpha; range=(0,Inf);

negative binomial model;
    outcome = ${y};
    dispersion = par Alpha;
% if offset:
    incidence = exp(${n} + regset BetaX + intres(draw=1, ref=u));
% else:
    incidence = exp(regset BetaX + intres(draw=1, ref=u));
% endif
% endif

starting values;
% for i in range(0, len(x)):
beta${i}	T	0
% endfor
sigmau	T	1
% if D == 'Normal':
sigma	T	1
% endif
% if D == 'Negative Binomial':
Alpha	F	1
% endif
;
'''

    latex = r'''
\begin{aligned} 
%if D == 'Normal':
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
%if D == 'Binomial':
 \mbox{${y}}_{i} & \sim \mbox{Binomial}(\mbox{${n}}_{i}, \pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%endif
%if D == 'Poisson':
 \mbox{${y}}_{i} & \sim \mbox{Poisson}(\pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%if offset:
\mbox{${n}}_{i} +
%endif
%endif
${mmulttex(x, r'\beta', 'i')} + u_{\mbox{${ID}}[i]}\\ 
u_{\mbox{${ID}}[i]} & \sim \mbox{N}(0,\sigma^2_u) \\ 
\end{aligned} 
'''
