# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class NLevelModAML(Template):
    'Fits random intercept models allowing one or more higher classifications (nested or crossed) in aML only. Allows for Normal, Poisson, binary, binomial or negative binomial responses.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal', 'Binomial', 'Poisson', 'Negative binomial', 'Probit', 'Logit' ]
    engines = ['aML']

    inputs = '''
NumLevs = Integer('Number of Classifications: ')
for i in range(0, int(NumLevs)):
    selstr = 'Classification ' + str(i + 1) + ': '
    context['C' + str(i + 1)] = IDVector(selstr)
    context['sigmau' + str(i + 1)] = ParamScalar()

ID = context['C'+str(int(NumLevs))]
y = DataVector('Response: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Binary', 'Poisson', 'Negative Binomial'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    link = Text('Specify link function: ', ['logit', 'probit'])
if D == 'Binary':
    link = Text('Specify link function: ', ['logit', 'probit'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
if D == 'Normal':
    sigma = ParamScalar()
if D == 'Negative Binomial':
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
beta = ParamVector(parents=[x], as_scalar=True)
'''

    amlscript = '''

option maximum number of residual draws = 200;

define regressor set BetaX;
var = \\
% for p in x:
${p} \\
% endfor
;

% for i in range(0, int(NumLevs)):
define normal distribution; dim=1;
    name=u${i};
% endfor

% if D == 'Normal':
define normal distribution; dim=1;
    name=e;

continuous model;
    outcome = ${y};
    model = regset BetaX
% for i in range(0, int(NumLevs)):
        + res(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
        + res(draw=_iid, ref=e);
% endif

% if D == 'Binomial':
binomial model;
    outcome = ${y};
    exposure = ${n};
% if link == 'logit':
    probability = logistic(regset BetaX
% for i in range(0, int(NumLevs)):
        + intres(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
);
% endif
% if link == 'probit':
    probability = normprob(regset BetaX
% for i in range(0, int(NumLevs)):
        + intres(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
);
% endif
% endif
% if D == 'Binary':
% if link == 'logit':
logit model;
% endif
% if link == 'probit':
probit model;
% endif
  outcome=use;
  model = regset BetaX
% for i in range(0, int(NumLevs)):
        + intres(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
;
% endif
% if D == 'Poisson':
poisson model;
    outcome = ${y};
% if offset:
    incidence = exp(${n} + regset BetaX
% for i in range(0, int(NumLevs)):
        + intres(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
);
% else:
    incidence = exp(regset BetaX
% for i in range(0, int(NumLevs)):
        + intres(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
);
% endif
% endif
% if D == 'Negative Binomial':
define parameter Alpha; range=(0,Inf);

negative binomial model;
    outcome = ${y};
    dispersion = par Alpha;
% if offset:
    incidence = exp(${n} + regset BetaX
% for i in range(0, int(NumLevs)):
        + intres(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
);
% else:
    incidence = exp(regset BetaX
% for i in range(0, int(NumLevs)):
        + intres(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
);
% endif
% endif


starting values;
% for i in range(0, len(x)):
beta${i}	T	0
% endfor
% for i in range(0, int(NumLevs)):
sigmau${i}	T	1
% endfor
% if D == 'Normal':
sigma	T	1
% endif
% if D == 'Negative Binomial':
Alpha	F	1
% endif
;
'''

    latex = r'''
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')} \\ 
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau
\end{aligned} 
'''
